import React from 'react'
import ReactDOM from 'react-dom'
import PropTypes from 'prop-types'

import Overlay from './Overlay'
import Content from './Content'
import baseClassName from './baseClassName'
import { classNamePropType } from './propTypes'
import { msToTimestamp, timeStampToMs } from './helpers'

class Modal extends React.Component {
  static classNames = {
    className: {
      base: `${baseClassName}__Modal`,
      onAfterOpen: `${baseClassName}__Modal--on-after-open`,
      onBeforeClose: `${baseClassName}__Modal--on-before-close`
    },
    overlayClassName: {
      base: `${baseClassName}__Overlay`,
      onAfterOpen: `${baseClassName}__Overlay--on-after-open`,
      onBeforeClose: `${baseClassName}__Overlay--on-before-close`
    },
    contentClassName: {
      base: `${baseClassName}__Content`,
      onAfterOpen: `${baseClassName}__Content--on-after-open`,
      onBeforeClose: `${baseClassName}__Content--on-before-close`
    }
  }

  static contextTypes = {
    modalStack: PropTypes.shape({
      rootNode: PropTypes.instanceOf(Element).isRequired,
      update: PropTypes.func.isRequired
    }).isRequired
  }

  static defaultProps = {
    styles: {},
    open: false,
    closeOnEsc: false,
    closeOnOverlayClick: true,
    closeTimeout: 0
  }

  static propTypes = {
    children: PropTypes.oneOfType([PropTypes.node, PropTypes.func]),
    className: classNamePropType,
    overlayClassName: classNamePropType,
    contentClassName: classNamePropType,
    ref: PropTypes.func,
    overlayRef: PropTypes.func,
    contentRef: PropTypes.func,
    styles: PropTypes.shape({
      overlay: PropTypes.object,
      content: PropTypes.object
    }),
    open: PropTypes.bool,
    closeOnEsc: PropTypes.bool,
    closeOnOverlayClick: PropTypes.bool,
    onClose: PropTypes.func,
    closeTimeout: PropTypes.number
  }

  state = {
    afterOpen: false,
    waitingClose: false
  }

  componentDidMount() {
    this.notifyModalStack()

    if (this.props.open) {
      this.requestOpen()
    }
  }

  componentWillReceiveProps(nextProps) {
    if (!this.props.open && nextProps.open) {
      this.requestOpen()
    } else if (this.props.open && !nextProps.open) {
      this.requestClose()
    }
  }

  componentDidUpdate() {
    this.notifyModalStack()
  }

  componentWillUnmount() {
    this.notifyModalStack()
  }

  notifyModalStack() {
    this.context.modalStack.update()
  }

  getClassName(classNameProp) {
    const className = this.props[classNameProp]
    const isRedefined = typeof className === 'object'

    const { base, onAfterOpen, onBeforeClose } = isRedefined
      ? className
      : Modal.classNames[classNameProp]

    let classNames = [base]

    if (onAfterOpen && this.state.afterOpen) {
      classNames.push(onAfterOpen)
    }
    if (onBeforeClose && this.state.waitingClose) {
      classNames.push(onBeforeClose)
    }
    if (!isRedefined) {
      classNames.push(className)
    }

    return classNames.join(' ')
  }

  requestOpen() {
    const { afterOpen, waitingClose } = this.state

    if (afterOpen && waitingClose) {
      clearTimeout(this.closeTimeout)
      this.setState({ waitingClose: false })
    } else {
      this.open()
    }
  }

  open() {
    this.setState({ isOpen: true }, () => {
      // FIXME
      setTimeout(() => {
        this.setState({ afterOpen: true })
      })
    })
  }

  requestClose() {
    const { closeTimeout } = this.props

    if (closeTimeout > 0) {
      const closeTimestamp = msToTimestamp(closeTimeout)

      this.setState({ waitingClose: true, closeTimestamp }, () => {
        this.closeTimeout = setTimeout(
          this.close,
          timeStampToMs(this.state.closeTimestamp)
        )
      })
    } else {
      this.close()
    }
  }

  close = () => {
    this.setState({
      waitingClose: false,
      isOpen: false,
      afterOpen: false,
      closeTimestamp: null
    })
  }

  renderChildren() {
    const { children } = this.props

    const renderedChildren =
      typeof children === 'function'
        ? children({
            close: this.handleOnCloseClick
          })
        : children

    if (!renderedChildren) {
      return null
    }

    return React.Children.only(renderedChildren)
  }

  render() {
    const {
      closeOnEsc,
      styles: { content: contentStyles, overlay: overlayStyles },
      closeOnOverlayClick
    } = this.props

    if (!this.state.isOpen) {
      return null
    }

    return ReactDOM.createPortal(
      <div className={this.getClassName('className')} ref={this.refNode}>
        <Overlay
          className={this.getClassName('overlayClassName')}
          style={overlayStyles}
          onClick={closeOnOverlayClick && this.handleOnCloseClick}
          ref={this.refOverlay}
        />
        <Content
          className={this.getClassName('contentClassName')}
          style={contentStyles}
          closeOnEsc={closeOnEsc}
          onRequestClose={this.handleOnCloseClick}
          ref={this.refContent}
        >
          {this.renderChildren()}
        </Content>
      </div>,
      this.context.modalStack.rootNode
    )
  }

  refNode = node => {
    this.node = node

    const { ref } = this.props

    if (ref) {
      ref(node)
    }
  }

  refOverlay = overlay => {
    this.overlay = overlay

    const { overlayRef } = this.props

    if (overlayRef) {
      overlayRef(overlay)
    }
  }

  refContent = content => {
    this.content = content

    const { contentRef } = this.props

    if (contentRef) {
      contentRef(content)
    }
  }

  handleOnCloseClick = () => {
    if (this.props.onClose) {
      this.props.onClose()
    }
  }
}

export default Modal
