import React from 'react'
import PropTypes from 'prop-types'
import warning from 'warning'
import baseClassName from './baseClassName'

class ModalStack extends React.Component {
  static propTypes = {
    children: PropTypes.node,
    rootNode: PropTypes.oneOfType([
      PropTypes.instanceOf(Element),
      PropTypes.string
    ]),
    unmountWhenEmpty: PropTypes.bool
  }

  static defaultProps = {
    rootNode: document.body,
    unmountWhenEmpty: false
  }

  static childContextTypes = {
    modalStack: PropTypes.shape({
      rootNode: PropTypes.instanceOf(Element)
    })
  }

  get portalNode() {
    return this._portalNode
  }

  set portalNode(node) {
    this._portalNode = node
  }

  get mountNode() {
    const { rootNode } = this.props

    if (this._mountNode) {
      return this._mountNode
    }

    if (typeof rootNode === 'string') {
      this._mountNode = document.querySelector(rootNode)
    } else if (rootNode) {
      this._mountNode = rootNode
    }

    return this._mountNode
  }

  constructor(props) {
    super(props)

    this.createPortalNode()
  }

  componentWillReceiveProps(nextProps) {
    warning(
      this.props.unmountWhenEmpty === nextProps.unmountWhenEmpty,
      'You cannot change "unmountWhenEmpty"'
    )
    warning(
      this.props.rootNode === nextProps.rootNode,
      'You cannot change "rootNode"'
    )
  }

  createPortalNode() {
    const node = document.createElement('div')
    node.classList.add(`${baseClassName}__Portal`)

    this.portalNode = node
  }

  mount() {
    if (this.portalNode.parentNode === this.mountNode) {
      return
    }

    this.mountNode.appendChild(this.portalNode)
  }

  unmount() {
    if (this.portalNode.parentNode !== this.mountNode) {
      return
    }

    this.mountNode.removeChild(this.portalNode)
  }

  componentDidMount() {
    if (!this.props.unmountWhenEmpty) {
      this.mount()
    }
  }

  componentWillUnmount() {
    this.unmount()
  }

  render() {
    return this.props.children
  }

  getChildContext() {
    return {
      modalStack: {
        rootNode: this.portalNode,
        update: this.handleUpdate,
        subscribe: this.handleSubscribe
      }
    }
  }

  handleUpdate = () => {
    const hasMountedModals = this.portalNode.hasChildNodes()

    if (this.props.unmountWhenEmpty) {
      if (hasMountedModals) {
        this.mount()
      } else {
        this.unmount()
      }
    }

    document.body.classList.toggle(
      `${baseClassName}__Body--opened`,
      hasMountedModals
    )
  }
}

export default ModalStack
